﻿using Dynamicweb.Ecommerce.LiveIntegration.Configuration;
using Dynamicweb.Ecommerce.Orders;
using Dynamicweb.Environment;
using Dynamicweb.Security.UserManagement;
using Dynamicweb.Security.UserManagement.Common.CustomFields;
using System;

namespace Dynamicweb.Ecommerce.LiveIntegration
{    
    /// <summary>
    /// UrlHandler to build up connection info for the ERP.
    /// </summary>
    public class UrlHandler
    {
        /// <summary>
        /// The synchronize root
        /// </summary>
        private static readonly object SyncRoot = new object();

        /// <summary>
        /// The instance
        /// </summary>
        private static UrlHandler _instance;

        /// <summary>
        /// Prevents a default instance of the <see cref="UrlHandler"/> class from being created.
        /// </summary>
        private UrlHandler()
        {
        }

        /// <summary>
        /// Gets the instance.
        /// </summary>
        /// <value>The instance.</value>
        public static UrlHandler Instance
        {
            get
            {
                if (_instance == null)
                {
                    lock (SyncRoot)
                    {
                        if (_instance == null)
                        {
                            _instance = new UrlHandler();
                        }
                    }
                }

                return _instance;
            }
        }

        /// <summary>
        /// Gets the URL cache key.
        /// </summary>
        /// <value>The URL cache key.</value>
        private string UrlCacheKey
        {
            get
            {
                return $"{Constants.AddInName}WebServiceURI";
            }
        }

        /// <summary>
        /// Clears the cached URL.
        /// </summary>
        public void ClearCachedUrl()
        {
            if (Context.Current.Items[UrlCacheKey] != null)
            {
                Context.Current.Items.Remove(UrlCacheKey);
            }
        }

        /// <summary>
        /// Gets the URL.
        /// </summary>
        /// <param name="url">The URL.</param>
        /// <returns>System.String.</returns>
        private string GetUrl(string url)
        {
            return url.Contains(";") ? url.Substring(0, url.IndexOf(";", StringComparison.Ordinal)) : url;
        }

        /// <summary>
        /// Gets the web service URL for the current context.
        /// </summary>
        /// <returns>System.String.</returns>
        public string GetWebServiceUrl()
        {
            string ret = string.Empty;
            if (Context.Current.Items[UrlCacheKey] != null)
            {
                ret = (string)Context.Current.Items[UrlCacheKey];
            }
            else
            {
                string multiLineUrlText = Settings.Instance.WebServiceURI;
                string[] urls = GetWebServiceUrls(multiLineUrlText);
                if (urls.Length > 0)
                {
                    if (multiLineUrlText.Contains(";"))
                    {
                        foreach (string url in urls)
                        {
                            string[] parts = url.Split(new char[] { ';' });
                            if (parts.Length > 1)
                            {
                                string uri = parts[0];
                                string fieldName = parts[1];
                                string fieldValue = parts[2];
                                if (!string.IsNullOrEmpty(fieldName))
                                {
                                    if (fieldName.StartsWith("User."))
                                    {
                                        if (TreatUserFields(fieldName, fieldValue))
                                        {
                                            ret = GetUrl(url);
                                            break;
                                        }
                                    }
                                    else if (fieldName.StartsWith("Order.") && GetCurrentOrder() != null)
                                    {
                                        foreach (OrderFieldValue ofv in GetCurrentOrder().OrderFieldValues)
                                        {
                                            if (ofv != null && ofv.OrderField != null && ofv.OrderField.SystemName == fieldName.Substring(6) && (string)ofv.Value == fieldValue)
                                            {
                                                ret = GetUrl(url);
                                                break;
                                            }
                                        }
                                    }
                                    else if (fieldName == "Session.Shop" && fieldValue == GetCurrentShopId())
                                    {
                                        ret = GetUrl(url);
                                        break;
                                    }
                                }
                            }
                        }
                    }
                    else
                    {
                        ret = urls[0];
                    }
                }

                Context.Current.Items[UrlCacheKey] = ret;
            }

            return ret;
        }

        /// <summary>
        /// Determines whether [is web service URL found].
        /// </summary>
        /// <returns><c>true</c> if [is web service URL found]; otherwise, <c>false</c>.</returns>
        public bool IsWebServiceUrlFound()
        {
            return !string.IsNullOrEmpty(GetWebServiceUrl());
        }

        /// <summary>
        /// Gets the current order.
        /// </summary>
        /// <returns>Order.</returns>
        private Order GetCurrentOrder()
        {
            Order ret = Common.Context.Cart;

            if (ret == null && !string.IsNullOrEmpty(Context.Current.Request["orderid"]))
            {
                ret = Order.GetOrderById(Context.Current.Request.GetString("orderid"));
            }

            return ret;
        }

        /// <summary>
        /// Gets the current shop identifier.
        /// </summary>
        /// <returns>System.String.</returns>
        private string GetCurrentShopId()
        {
            string ret = string.Empty;

            if (Dynamicweb.Frontend.PageView.Current() != null && Dynamicweb.Frontend.PageView.Current().Area != null)
            {
                ret = Dynamicweb.Frontend.PageView.Current().Area.EcomShopId;
            }
            else if (!string.IsNullOrEmpty(Context.Current.Request["shopid"]))
            {
                ret = Context.Current.Request["shopid"];
            }
            else if (!string.IsNullOrEmpty((string)Context.Current.Items["shopid"]))
            {
                ret = (string)Context.Current.Items["shopid"];
            }

            return ret;
        }

        /// <summary>
        /// Gets the web service urls.
        /// </summary>
        /// <param name="multilineUrlText">The multiline URL text.</param>
        /// <returns>System.String[].</returns>
        private string[] GetWebServiceUrls(string multilineUrlText)
        {
            string[] ret;

            if (!string.IsNullOrEmpty(multilineUrlText))
            {
                ret = new string[] { multilineUrlText };
                if (multilineUrlText.Contains("\r\n") || multilineUrlText.Contains("\n"))
                {
                    ret = multilineUrlText.Split(new string[] { "\r\n", "\n" }, StringSplitOptions.RemoveEmptyEntries);
                }
            }
            else
            {
                ret = new string[] { };
            }

            return ret;
        }

        /// <summary>
        /// Treats the user fields.
        /// </summary>
        /// <param name="fieldName">Name of the field.</param>
        /// <param name="fieldValue">The field value.</param>
        /// <returns><c>true</c> if the user field matched the passed <c>fieldName</c>, <c>false</c> otherwise.</returns>
        private bool TreatUserFields(string fieldName, string fieldValue)
        {
            User user = User.GetCurrentExtranetUser();

            if (user == null)
            {
                return false;
            }

            if ((fieldName == "User.Company" && user.Company == fieldValue) || (fieldName == "User.Department" && user.Department == fieldValue))
            {
                return true;
            }
            else
            {
                foreach (CustomFieldValue cfv in user.CustomFieldValues)
                {
                    // fieldName.Substring: Remove the "User." prefix to
                    // obtain system name only
                    if (cfv != null
                        && cfv.CustomField != null
                        && cfv.CustomField.SystemName == fieldName.Substring(5)
                        && (string)cfv.Value == fieldValue)
                    {
                        return true;
                    }
                }
            }

            return false;
        }
    }
}